@@ -28,6 +28,21 @@ from .common import COMPRESSIONS as _COMPRESSIONS
 from .data_format.elf import from_file as _data_format_elf_from_file
 
 
+def _format_compression(compression, compression_info):
+    info = None
+
+    if compression == 'heatshrink':
+        if compression_info:
+            window_sz2 = compression_info['window-sz2']
+            lookahead_sz2 = compression_info['lookahead-sz2']
+            info = f'window-sz2: {window_sz2}, lookahead-sz2: {lookahead_sz2}'
+
+    if info:
+        compression += f' ({info})'
+
+    return compression
+
+
 def parse_integer(option, value):
     try:
         return int(value, 0)
@@ -215,6 +230,13 @@ def data_format_args(args):
     }
 
 
+def heatshrink_args(args):
+    return {
+        'heatshrink_window_sz2': args.heatshrink_window_sz2,
+        'heatshrink_lookahead_sz2': args.heatshrink_lookahead_sz2
+    }
+
+
 def print_successful(filename, start_time):
     print("Successfully created '{}' in {}!".format(
         filename,
@@ -233,6 +255,7 @@ def _do_create_patch(args):
                            match_score=args.match_score,
                            match_block_size=args.match_block_size,
                            use_mmap=not args.no_mmap,
+                           **heatshrink_args(args),
                            **data_format_args(args))
     print_successful(args.patchfile, start_time)
 
@@ -250,6 +273,7 @@ def _do_create_patch_in_place(args):
                            args.segment_size,
                            args.minimum_shift_size,
                            use_mmap=not args.no_mmap,
+                           **heatshrink_args(args),
                            **data_format_args(args))
     print_successful(args.patchfile, start_time)
 
@@ -349,6 +373,7 @@ def _patch_info_sequential(detailed,
                            fsize,
                            patch_size,
                            compression,
+                           compression_info,
                            dfpatch_size,
                            data_format,
                            dfpatch_info,
@@ -365,6 +390,7 @@ def _patch_info_sequential(detailed,
     size_data_ratio = _format_ratio(number_of_size_bytes, number_of_data_bytes)
     patch_to_ratio = _format_ratio(patch_size, to_size)
     diff_extra_ratio = _format_ratio(number_of_diff_bytes, number_of_extra_bytes)
+    compression = _format_compression(compression, compression_info)
 
     if diff_sizes:
         mean_diff_size = fsize(int(mean(diff_sizes)))
@@ -413,6 +439,7 @@ def _patch_info_sequential(detailed,
 def _patch_info_in_place(fsize,
                          patch_size,
                          compression,
+                         compression_info,
                          memory_size,
                          segment_size,
                          from_shift_size,
@@ -420,6 +447,7 @@ def _patch_info_in_place(fsize,
                          to_size,
                          segments):
     patch_to_ratio = _format_ratio(patch_size, to_size)
+    compression = _format_compression(compression, compression_info)
 
     print('Type:               in-place')
     print('Patch size:         {}'.format(fsize(patch_size)))
@@ -452,8 +480,10 @@ def _patch_info_in_place(fsize,
 def _patch_info_hdiffpatch(fsize,
                            patch_size,
                            compression,
+                           compression_info,
                            to_size):
     patch_to_ratio = _format_ratio(patch_size, to_size)
+    compression = _format_compression(compression, compression_info)
 
     print('Type:               hdiffpatch')
     print('Patch size:         {}'.format(fsize(patch_size)))
@@ -521,6 +551,19 @@ def add_data_format_args(subparser):
         help='To file data address ranges.')
 
 
+def add_heatshrink_args(subparser):
+    subparser.add_argument(
+        '--heatshrink-window-sz2',
+        default=8,
+        type=int,
+        help='Heatshrink window sz2 setting (default: %(default)s).')
+    subparser.add_argument(
+        '--heatshrink-lookahead-sz2',
+        default=7,
+        type=int,
+        help='Heatshrink lookahead sz2 setting (default: %(default)s).')
+
+
 def _main():
     parser = argparse.ArgumentParser(description='Binary delta encoding utility.')
 
@@ -580,6 +623,7 @@ def _main():
     subparser.add_argument('--no-mmap',
                            action='store_true',
                            help='Do not use mmap.')
+    add_heatshrink_args(subparser)
     add_data_format_args(subparser)
     subparser.add_argument('fromfile', help='From file.')
     subparser.add_argument('tofile', help='To file.')
@@ -613,6 +657,7 @@ def _main():
     subparser.add_argument('--no-mmap',
                            action='store_true',
                            help='Do not use mmap.')
+    add_heatshrink_args(subparser)
     add_data_format_args(subparser)
     subparser.add_argument('fromfile', help='From file.')
     subparser.add_argument('tofile', help='To file.')
